//  FluxboxEvent.hh for fluxbox.
//
//  Copyright (c) 2002 Steve Cooper, stevencooper@isomedia.com
//  Copyright (c) 1998-2000 by John Kennis, jkennis@chello.nl
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// (See the included file COPYING / GPL-2.0)
//

#ifndef __FLUXBOXEVENT_HH
#define __FLUXBOXEVENT_HH

#include <X11/Xlib.h>

class BaseDisplay;

class FluxboxRawEventHandler
{
  public:
    virtual void handleFluxboxEvents(XEvent& event) = 0;
    virtual void handleCreateNotify(XEvent& event) = 0;
    virtual void handleDestroyNotify(XEvent& event) = 0;
    virtual void handleMapNotify(XEvent& event) = 0;
    virtual void handleUnmapNotify(XEvent& event) = 0;
    virtual void handleReparentNotify(XEvent& event) = 0;
    virtual void handleConfigureNotify(XEvent& event) = 0;
    virtual void handlePropertyNotify(XEvent& event) = 0;
};

class FluxboxCookedEventHandler
{
  public:
    virtual void onStartup() = 0;
    virtual void onWindowCreate(Window w, Window parent) = 0;
    virtual void onWindowDestroy(Window w) = 0;
    virtual void onWindowAdd(Window w, int workspace) = 0;
    virtual void onWindowDelete(Window w) = 0;
    virtual void onWindowChangeAttributes(Window w) = 0;
    virtual void onWindowChangeParent(Window w, Window parent) = 0;
    virtual void onWindowConfigure(Window w) = 0;
    virtual void onWindowMap(Window w) = 0;
    virtual void onWindowUnmap(Window w) = 0;
    virtual void onWindowFocus(Window w) = 0;
    virtual void onWindowRaise(Window w) = 0;
    virtual void onWindowLower(Window w) = 0;
    virtual void onWorkspaceChangeCurrent(int workspace) = 0;
    virtual void onWorkspaceChangeCount(int nWorkspaces) = 0;
};

class FluxboxEventCooker : public FluxboxRawEventHandler
{
  public:
    FluxboxEventCooker(BaseDisplay& baseDisplay_, FluxboxCookedEventHandler& cookedHandler_);
    virtual ~FluxboxEventCooker();

  private:
    virtual void handleFluxboxEvents(XEvent& event);
    virtual void handleCreateNotify(XEvent& event);
    virtual void handleDestroyNotify(XEvent& event);
    virtual void handleMapNotify(XEvent& event);
    virtual void handleUnmapNotify(XEvent& event);
    virtual void handleReparentNotify(XEvent& event);
    virtual void handleConfigureNotify(XEvent& event);
    virtual void handlePropertyNotify(XEvent& event);

  protected:
    BaseDisplay&               baseDisplay;
    FluxboxCookedEventHandler& cookedHandler;

    bool GetCardinalProperty(Window w, Atom& atom, long length, long* values);

  private:
    bool initialized;
};

#endif // __FLUXBOXEVENT_HH
