// FluxspaceDisplay.hh for fluxspace - a workspace manager for fluxbox
//
//  Copyright (c) 2002 Steve Cooper, stevencooper@isomedia.com
//  Copyright (c) 1998-2000 by John Kennis, jkennis@chello.nl
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// (See the included file COPYING / GPL-2.0)
//

#ifndef __FLUXSPACEDISPLAY_HH
#define __FLUXSPACEDISPLAY_HH

#include "BaseDisplay.hh"

class FluxboxRawEventHandler;

class PythonInterface;

class FluxspaceDisplay : public BaseDisplay
{
  public:
    class Exception
    {
      public:
        Exception(char* description_)
          : description(description)
        {
        }
        char* description;
    };

    FluxspaceDisplay(const char* displayName = NULL, bool verbose_ = false);
    virtual ~FluxspaceDisplay();

    ScreenInfo& getCurrentScreenInfo()
    {
        if (!currentScreenInfo)
            throw Exception("No screen info");
        return *currentScreenInfo;
    }

    Window getRootWindow() const
    {
        return const_cast<FluxspaceDisplay*>(this)->getCurrentScreenInfo().getRootWindow();
    }

    Window getWindow() const
    {
        return win;
    }

    void go(FluxboxRawEventHandler& rawEventHandler_);

  protected:
    virtual void process_event(XEvent *);
    virtual void reconfigure();
    virtual Bool handleSignal(int);

  private:
	Window		win;
    ScreenInfo* currentScreenInfo;
    Atom        wmDeleteWindow;
    bool        verbose;
    FluxboxRawEventHandler* rawEventHandler;

    Window MakeWindow();
};

#endif // __FLUXSPACEDISPLAY_HH
