//  PythonInterface.cc for fluxspace.
//
//  Copyright (c) 2002 Steve Cooper, stevencooper@isomedia.com
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// (See the included file COPYING / GPL-2.0)
//

#include <string>
#include "PythonInterface.hh"

class PythonInterfaceObject
{
  public:
    PythonInterfaceObject(PyObject* pObj)
      : next(NULL),
        object(pObj),
        methInitialize(NULL),
        methWindowCreate(NULL),
        methWindowDestroy(NULL),
        methWindowAdd(NULL),
        methWindowRemove(NULL),
        methWindowChangeAttributes(NULL),
        methWindowFocus(NULL),
        methWindowRaise(NULL),
        methWindowLower(NULL),
        methWindowConfigure(NULL),
        methWindowChangeParent(NULL),
        methWindowMap(NULL),
        methWindowUnmap(NULL),
        methWorkspaceChangeCount(NULL),
        methWorkspaceIn(NULL),
        methWorkspaceOut(NULL)
    {
        methInitialize             = new MyPyMethod(object, "Initialize",             true);
        methWindowCreate           = new MyPyMethod(object, "WindowCreate",           true);
        methWindowDestroy          = new MyPyMethod(object, "WindowDestroy",          true);
        methWindowAdd              = new MyPyMethod(object, "WindowAdd",              true);
        methWindowRemove           = new MyPyMethod(object, "WindowRemove",           true);
        methWindowChangeAttributes = new MyPyMethod(object, "WindowChangeAttributes", true);
        methWindowFocus            = new MyPyMethod(object, "WindowFocus",            true);
        methWindowRaise            = new MyPyMethod(object, "WindowRaise",            true);
        methWindowLower            = new MyPyMethod(object, "WindowLower",            true);
        methWindowConfigure        = new MyPyMethod(object, "WindowConfigure",        true);
        methWindowChangeParent     = new MyPyMethod(object, "WindowChangeParent",     true);
        methWindowMap              = new MyPyMethod(object, "WindowMap",              true);
        methWindowUnmap            = new MyPyMethod(object, "WindowUnmap",            true);
        methWorkspaceChangeCount   = new MyPyMethod(object, "WorkspaceChangeCount",   true);
        methWorkspaceIn            = new MyPyMethod(object, "WorkspaceIn",            true);
        methWorkspaceOut           = new MyPyMethod(object, "WorkspaceOut",           true);
    }

    virtual ~PythonInterfaceObject()
    {
        delete methInitialize;
        delete methWindowCreate;
        delete methWindowDestroy;
        delete methWindowAdd;
        delete methWindowRemove;
        delete methWindowChangeAttributes;
        delete methWindowFocus;
        delete methWindowRaise;
        delete methWindowLower;
        delete methWindowConfigure;
        delete methWindowChangeParent;
        delete methWindowMap;
        delete methWindowUnmap;
        delete methWorkspaceChangeCount;
        delete methWorkspaceIn;
        delete methWorkspaceOut;
    }

    PythonInterfaceObject* next;
    static PythonInterfaceObject* head;

    static void Append(PythonInterfaceObject* newObject)
    {
        if (head == NULL)
        {
            head = newObject;
        }
        else
        {
            PythonInterfaceObject* object = head;
            while (true)
            {
                if (object->next == NULL)
                {
                    object->next = newObject;
                    break;
                }
                object = object->next;
            }
        }
    }

    static void PurgeAll()
    {
        PythonInterfaceObject* object = head;
        while (object != NULL)
        {
            PythonInterfaceObject* next = object->next;
            delete object;
            object = next;
        }
    }

    void Initialize()
    {
        methInitialize->Call();
    }

    void WindowCreate(Window window, Window parent)
    {
        methWindowCreate->BeginArgs(2);
        methWindowCreate->AddArgInt(window);
        methWindowCreate->AddArgInt(parent);
        methWindowCreate->Call();
    }

    void WindowDestroy(Window window)
    {
        methWindowDestroy->BeginArgs(1);
        methWindowDestroy->AddArgInt(window);
        methWindowDestroy->Call();
    }

    void WindowAdd(Window window, int workspace)
    {
        methWindowAdd->BeginArgs(2);
        methWindowAdd->AddArgInt(window);
        methWindowAdd->AddArgInt(workspace);
        methWindowAdd->Call();
    }

    void WindowRemove(Window window)
    {
        methWindowRemove->BeginArgs(1);
        methWindowRemove->AddArgInt(window);
        methWindowRemove->Call();
    }

    void WindowChangeAttributes(Window window)
    {
        methWindowChangeAttributes->BeginArgs(1);
        methWindowChangeAttributes->AddArgInt(window);
        methWindowChangeAttributes->Call();
    }

    void WindowFocus(Window window)
    {
        methWindowFocus->BeginArgs(1);
        methWindowFocus->AddArgInt(window);
        methWindowFocus->Call();
    }

    void WindowRaise(Window window)
    {
        methWindowRaise->BeginArgs(1);
        methWindowRaise->AddArgInt(window);
        methWindowRaise->Call();
    }

    void WindowLower(Window window)
    {
        methWindowLower->BeginArgs(1);
        methWindowLower->AddArgInt(window);
        methWindowLower->Call();
    }

    void WindowConfigure(Window window)
    {
        methWindowConfigure->BeginArgs(1);
        methWindowConfigure->AddArgInt(window);
        methWindowConfigure->Call();
    }

    void WindowChangeParent(Window window, Window parent)
    {
        methWindowChangeParent->BeginArgs(2);
        methWindowChangeParent->AddArgInt(window);
        methWindowChangeParent->AddArgInt(parent);
        methWindowChangeParent->Call();
    }

    void WindowMap(Window window)
    {
        methWindowMap->BeginArgs(1);
        methWindowMap->AddArgInt(window);
        methWindowMap->Call();
    }

    void WindowUnmap(Window window)
    {
        methWindowUnmap->BeginArgs(1);
        methWindowUnmap->AddArgInt(window);
        methWindowUnmap->Call();
    }

    void WorkspaceChangeCount(int nWorkspaces)
    {
        methWorkspaceChangeCount->BeginArgs(1);
        methWorkspaceChangeCount->AddArgInt(nWorkspaces);
        methWorkspaceChangeCount->Call();
    }

    void WorkspaceIn(int workspace)
    {
        methWorkspaceIn->BeginArgs(1);
        methWorkspaceIn->AddArgInt(workspace);
        methWorkspaceIn->Call();
    }

    void WorkspaceOut(int workspace)
    {
        methWorkspaceOut->BeginArgs(1);
        methWorkspaceOut->AddArgInt(workspace);
        methWorkspaceOut->Call();
    }

  private:
    MyPyObjectRef object;   // Use "Ref" to increment reference count
    MyPyMethod* methInitialize;
    MyPyMethod* methWindowCreate;
    MyPyMethod* methWindowDestroy;
    MyPyMethod* methWindowAdd;
    MyPyMethod* methWindowRemove;
    MyPyMethod* methWindowChangeAttributes;
    MyPyMethod* methWindowFocus;
    MyPyMethod* methWindowRaise;
    MyPyMethod* methWindowLower;
    MyPyMethod* methWindowConfigure;
    MyPyMethod* methWindowChangeParent;
    MyPyMethod* methWindowMap;
    MyPyMethod* methWindowUnmap;
    MyPyMethod* methWorkspaceChangeCount;
    MyPyMethod* methWorkspaceIn;
    MyPyMethod* methWorkspaceOut;
};

PythonInterfaceObject* PythonInterfaceObject::head = NULL;

class PythonInterfaceModule
{
  public:
    PythonInterfaceModule::PythonInterfaceModule(const string& name_, PyObject* config_)
      : name(name_),
        next(NULL),
        module(name_),
        config(config_),
        FSInitialize(NULL)
    {
        FSInitialize = new MyPyFunctionPtr(module.dict.GetItemString("fluxlet_main"));
    }

    PythonInterfaceModule::~PythonInterfaceModule()
    {
        delete FSInitialize;
    }

    void Initialize()
    {
        FSInitialize->BeginArgs(1);
        FSInitialize->AddArgObject(config);
        FSInitialize->Call();
    }

    static void Append(PythonInterfaceModule* newModule)
    {
        if (head == NULL)
        {
            head = newModule;
        }
        else
        {
            PythonInterfaceModule* module = head;
            while (true)
            {
                if (module->next == NULL)
                {
                    module->next = newModule;
                    break;
                }
                module = module->next;
            }
        }
    }

    static void PurgeAll()
    {
        PythonInterfaceModule* module = head;
        while (module != NULL)
        {
            PythonInterfaceModule* next = module->next;
            delete module;
            module = next;
        }
    }

    string name;
    PythonInterfaceModule* next;
    static PythonInterfaceModule* head;

  private:
    MyPyModule module;
    MyPyObject config;
    MyPyFunctionPtr* FSInitialize;
};

PythonInterfaceModule* PythonInterfaceModule::head = NULL;

PythonInterface::PythonInterface()
{
}

PythonInterface::~PythonInterface()
{
    PythonInterfaceModule::PurgeAll();
    PythonInterfaceObject::PurgeAll();
}

void PythonInterface::AddModule(const string& name, PyObject* config)
{
    PythonInterfaceModule::Append(new PythonInterfaceModule(name, config));
}

void PythonInterface::AddHandler(PyObject* object)
{
    PythonInterfaceObject::Append(new PythonInterfaceObject(object));
}

void PythonInterface::Initialize()
{
    // Initialize modules
    for (PythonInterfaceModule* module = PythonInterfaceModule::head; module != NULL; module = module->next)
        module->Initialize();

    // Initialize objects added by module initializations
    for (PythonInterfaceObject* object = PythonInterfaceObject::head; object != NULL; object = object->next)
        object->Initialize();
}

void PythonInterface::WindowCreate(Window window, Window parent)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WindowCreate(window, parent);
        }
        catch(MyPyException e)
        {
            printf("WindowCreate failed: \"%s\"\n", e.text.c_str());
        }
    }
}

void PythonInterface::WindowDestroy(Window window)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WindowDestroy(window);
        }
        catch(MyPyException e)
        {
            printf("WindowDestroy failed: \"%s\"\n", e.text.c_str());
        }
    }
}

void PythonInterface::WindowAdd(Window window, int workspace)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WindowAdd(window, workspace);
        }
        catch(MyPyException e)
        {
            printf("WindowAdd failed: \"%s\"\n", e.text.c_str());
        }
    }
}

void PythonInterface::WindowRemove(Window window)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WindowRemove(window);
        }
        catch(MyPyException e)
        {
            printf("WindowRemove failed: \"%s\"\n", e.text.c_str());
        }
    }
}

void PythonInterface::WindowChangeAttributes(Window window)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WindowChangeAttributes(window);
        }
        catch(MyPyException e)
        {
            printf("WindowChangeAttributes failed: \"%s\"\n", e.text.c_str());
        }
    }
}

void PythonInterface::WindowFocus(Window window)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WindowFocus(window);
        }
        catch(MyPyException e)
        {
            printf("WindowFocus failed: \"%s\"\n", e.text.c_str());
        }
    }
}

void PythonInterface::WindowRaise(Window window)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WindowRaise(window);
        }
        catch(MyPyException e)
        {
            printf("WindowRaise failed: \"%s\"\n", e.text.c_str());
        }
    }
}

void PythonInterface::WindowLower(Window window)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WindowLower(window);
        }
        catch(MyPyException e)
        {
            printf("WindowLower failed: \"%s\"\n", e.text.c_str());
        }
    }
}

void PythonInterface::WindowConfigure(Window window)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WindowConfigure(window);
        }
        catch(MyPyException e)
        {
            printf("WindowConfigure failed: \"%s\"\n", e.text.c_str());
        }
    }
}

void PythonInterface::WindowChangeParent(Window window, Window parent)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WindowChangeParent(window, parent);
        }
        catch(MyPyException e)
        {
            printf("WindowChangeParent failed: \"%s\"\n", e.text.c_str());
        }
    }
}

void PythonInterface::WindowMap(Window window)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WindowMap(window);
        }
        catch(MyPyException e)
        {
            printf("WindowMap failed: \"%s\"\n", e.text.c_str());
        }
    }
}

void PythonInterface::WindowUnmap(Window window)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WindowUnmap(window);
        }
        catch(MyPyException e)
        {
            printf("WindowUnmap failed: \"%s\"\n", e.text.c_str());
        }
    }
}

void PythonInterface::WorkspaceChangeCount(int nWorkspaces)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WorkspaceChangeCount(nWorkspaces);
        }
        catch(MyPyException e)
        {
            printf("WorkspaceChangeCount failed: \"%s\"\n", e.text.c_str());
        }
    }
}


void PythonInterface::WorkspaceIn(int workspace)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WorkspaceIn(workspace);
        }
        catch(MyPyException e)
        {
            printf("WorkspaceIn failed: \"%s\"\n", e.text.c_str());
        }
    }
}

void PythonInterface::WorkspaceOut(int workspace)
{
    for (PythonInterfaceObject* object = PythonInterfaceObject::head;
         object != NULL;
         object = object->next)
    {
        try
        {
            object->WorkspaceOut(workspace);
        }
        catch(MyPyException e)
        {
            printf("WorkspaceOut failed: \"%s\"\n", e.text.c_str());
        }
    }
}
