#include <string>
#include <sys/types.h>
#include <Python.h>
#include <X11/Xlib.h>

using namespace std;

class ControllerImplementation;

class Error
{
  public:
    Error(const char* msg_, ...);

    string msg;
};

class InternalError : public Error
{
  public:
    InternalError(const char* msg_, ...);
};

class ErrorNoDisplay : public Error
{
  public:
    ErrorNoDisplay() : Error("X Display has not been established") {}
};

class ErrorControllerExists : public Error
{
  public:
    ErrorControllerExists() : Error("A Controller already exists") {}
};

class ErrorNoController : public Error
{
  public:
    ErrorNoController() : Error("A Controller must be created") {}
};

class ErrorEventLoopRunning : public Error
{
  public:
    ErrorEventLoopRunning() : Error("Event loop has already been started") {}
};

class ErrorBadController : public Error
{
  public:
    ErrorBadController() : Error("Controller has not been properly initialized") {}
};

class ErrorNoWindow : public Error
{
  public:
    ErrorNoWindow() : Error("Window does not exist") {}
};

class ErrorWindowProperty : public Error
{
  public:
    ErrorWindowProperty(const char* propName, const char* op)
      : Error("Unable to access \"%s\" window property for \"%s\" operation", propName, op)
    {}
};

class Controller
{
  public:
    Controller(const char* displayName = NULL, bool verbose = false) throw(Error);
    virtual ~Controller();
    void RegisterFluxlet(const char* moduleName, PyObject* config) throw(Error);
    void StartEvents() throw(Error);

    // debug-only message
    void Trace(const char* message);
    // severity: 0=information, 1=warning, 2=error
    void Message(int severity, const char* message);

    bool verbose;
};

class FluxletHelper
{
  public:
    FluxletHelper(const char* name) throw(Error);
    virtual ~FluxletHelper();
    void AddHandler(PyObject* object) throw(Error);
    void SetRoot(const char* fname,
                 const char* bgcolor = NULL,
                 bool        scale   = false,
                 bool        center  = false,
                 bool        fit     = false,
                 bool        mirror  = false) throw(Error);
    // debug-only message
    void Trace(const char* message);
    // severity: 0=information, 1=warning, 2=error
    void Message(int severity, const char* message);

    bool   verbose;
    string name;
};
